
/*************************************************************************************
 Arnav Kumar | M259 W23
 
 Project 3: 3D Graph Visualization
 
 Adapted from Jiaxin Wu's "MoMA Visualization" (https://vislab.mat.ucsb.edu/2022/p3/JiaxinWu/Project3_JiaxinWu.html), 
 Russell Liu's "Tesla Stock Checkouts" (https://vislab.mat.ucsb.edu/2022/p2/RussellLiu/Project2_Russell.html), 
 my previous Project 2 and those credited therein, and course references from the MAT259 course.
 
 Data: CBL-VR Video Data.
 
 Description: Insights extracted from a research video with the purpose of demoing Azure's Video Indexer 
 and later embedded with the Universal Sentence Encoder and PCA.
 
 Range: 0 min - 42 min
 
 Usage: 
 1. Hover over the data points for more info
 2. Press the buttons on the screen to filter by insightType
 3. Press '~' to cycle background colors
 4. Press '>' to cycle through animation speeds
 5. Press '|' to cycle through fields of view
 6. Press '+' to capture and save a screenshot of the timeline
 7. Press '!' to toggle presentation mode
 8. Navigate Peasy Cam to explore the environment.
 
 *************************************************************************************/
color colorDetermine(String subject){
  color c = #FFFFFF;
  if(subject.equals("transcript")){
    c = #0F5FFF;
  }
  else if(subject.equals("ocr")){
    c = #D39FFF;
  }
  else if(subject.equals("keywords")){
    c = #FF4336;
  }
  else if(subject.equals("topics")){
    c = #EF7350;
  }
  else if(subject.equals("labels")){
    c = #E57973;
  }
  else if(subject.equals("namedLocations")){
    c = #EF9D9A;
  }
  else if(subject.equals("namedPeople")){
    c = #FFDDD2;
  }
  else if(subject.equals("audioEffects")){
    c = #FFEBEB;
  }
    else if(subject.equals("sentiments")){
    c = #FF00EB;
  }
    else if(subject.equals("emotions")){
    c = #FFEB0B;
  }
  return c;
}

import controlP5.*;
import peasy.*;
import java.util.Random;
ControlP5 cp5;
PeasyCam cam;

Table table;
int numRows;
int drawspeed = 10;
int background = 0;
int offset = -500;
PFont font;
float fov;
float cameraZ;
float[] randoms;
boolean presentationMode = false;
boolean Transcript;
boolean OCR;
boolean Keywords;
boolean Topics;
boolean Labels;
boolean namedLocations;
boolean namedPeople;
boolean audioEffects;
boolean Sentiments;
boolean Emotions;
boolean Point;
boolean Line;

ArrayList<insight> insights = new ArrayList<insight>();

void setup(){
  //size(800, 600, P3D);
  size(1440, 820, P3D);
  smooth(6);
  pixelDensity(displayDensity());
  font = createFont("AmericanTypewriter-CondensedLight", 30);
  //background(0);
    // Set the angle of view - like changing the lens on a camera PI/2 (wide angle); PI/6 normal lens; PI/12 (telephoto)
  fov = (PI / 10) + (PI / 4); // field of view try between PI/2 to PI/10
  cameraZ = (height/2.0) / tan(PI/6);
  //perspective(field of view in radians; aspect/ratio of width to height; zNear - Z position of nearest clipping, zFar - Z position of farthest clipping)
  //default values are : perspective(PI/3.0, width/height, cameraZ/10.0, cameraZ*10.0) where cameraZ is ((height/2.0) / tan(PI*60.0/360.0));
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*10.0); // if using 0.001 instead of cameraZ/10.0 so that all things close to the screen can be seen - get noise
  cp5 = new ControlP5(this);
  cam = new PeasyCam(this, 200); 
  
  table = loadTable("ArnavKumarMAT259AzureVideoIndexerInsightsEmbeddings.csv", "header");
  numRows = table.getRowCount();
  //numRows = 1000;
  
  for (int i=0; i<numRows;i++){
    String insightType = table.getString(i, 0);
    String insight = table.getString(i, 1);
    float confidence = table.getFloat(i, 2);
    String stringstart = table.getString(i, 3);
    String stringend = table.getString(i, 4);
    float start = 0;
    float end = 0;
    float x = table.getFloat(i, 5);
    float y = table.getFloat(i, 6);
    float z = table.getFloat(i, 7);
    insight newinsight = new insight(insightType, insight, confidence, start, 
  end, stringstart, stringend, x * 100, y * 100, z * 100);
  
    float[] startfloatlist = float(split(stringstart, ':'));
  
    float[] endfloatlist = float(split(stringend, ':'));
  
    newinsight.start += (startfloatlist[1] * 60) + startfloatlist[2];
  
    newinsight.end = (endfloatlist[1] * 60) + endfloatlist[2];
  
    insights.add(newinsight);
  }
  print(insights.size());
  
  Transcript = false;
  OCR = false;
  Keywords = false;
  Topics = false;
  Labels = false;
  namedLocations = false;
  namedPeople = false;
  audioEffects = false;
  Sentiments = false;
  Emotions = false;
  Point = true;
  Line = true;
  int X=10;
  
  cp5.addTextfield("Menu")
    .setPosition(0, 720)
    .setSize(1440, 100)
    .setFocus(true)
    .setColor(color(0))
    .setColorBackground(color(200))
    .setColorActive(color(200))
    .setColorForeground(color(200))
    .setCaptionLabel("")
    ;
  
  cp5.addToggle("Transcript").setPosition(1360, X).setColorLabel(100)
  .setColorActive(colorDetermine("transcript")).setColorBackground(colorDetermine("transcript"));
  X += 60;
  cp5.addToggle("OCR").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("ocr"))
  .setColorBackground(colorDetermine("ocr"));
  X += 60;
  cp5.addToggle("Keywords").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("keywords"))
  .setColorBackground(colorDetermine("keywords"));
  X += 60;
  cp5.addToggle("Topics").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("topics"))
  .setColorBackground(colorDetermine("topics"));
  X += 60;
  cp5.addToggle("Labels").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("labels"))
  .setColorBackground(colorDetermine("labels"));
  X += 60;
  cp5.addToggle("namedLocations").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("namedLocations"))
  .setColorBackground(colorDetermine("namedLocations"));
  X += 60;
  cp5.addToggle("namedPeople").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("namedPeople"))
  .setColorBackground(colorDetermine("namedPeople"));
  X += 60;
  cp5.addToggle("audioEffects").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("audioEffects"))
  .setColorBackground(colorDetermine("audioEffects"));
  X += 60;
  cp5.addToggle("Sentiments").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("sentiments"))
  .setColorBackground(colorDetermine("sentiments"));
  X += 60;
  cp5.addToggle("Emotions").setPosition(1360, X).setColorLabel(100).setColorActive(colorDetermine("emotions"))
  .setColorBackground(colorDetermine("emotions"));
  X += 60;
  cp5.addToggle("Point").setPosition(1360, X).setColorLabel(100);
  X += 60;
  cp5.addToggle("Line").setPosition(1360, X).setColorLabel(100);
    
  /* cp5.addTextarea("title")
    .setPosition(height - 25, 10)
    .setSize(330,70)
    .setLineHeight(14)
    .setColor(color(0))
    .setColorBackground(color(0, 150))
    .setColorForeground(color(0, 150));
    ; */
    
  cp5.addTextlabel("Hover")
    .setText("Hover over the data points for more info")
    .setPosition(800, 730)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
    
  cp5.addTextlabel("~")
    .setText("Press '~' to cycle background colors")
    .setPosition(800, 755)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
    
  cp5.addTextlabel(">")
    .setText("Press '>' to cycle through animation speeds")
    .setPosition(800, 780)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
    
  cp5.addTextlabel("|")
    .setText("Press '|' to cycle through fields of view")
    .setPosition(1100, 730)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
    
  cp5.addTextlabel("+")
    .setText("Press '+' to capture and save a screenshot of the timeline")
    .setPosition(1100, 755)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
    
  cp5.addTextlabel("!")
    .setText("Press '!' to toggle presentation mode")
    .setPosition(1100, 780)
    .setColorValue(1)
    .setFont(createFont("Helvetica", 6))
    ;
  cp5.addTextlabel("texttitle")
    .setText("Azure Video Indexer Insights From CBL-VR's Research\n\nA 3D Graph By Arnav Kumar")
    .setPosition(10, 730)
    .setColorValue(1)
    .setFont(createFont("Zapfino", 6))
    ;
    
  cp5.addTextlabel("description")
    .setText("A 3D graph built on extracted machine learning insights\nfrom a research video collected under the CBL-VR project,\nbased in UCSB's GGSE. Nodes are connected based on\nwhere the insights overlap with each other during the video.")
    .setPosition(475, 730)
    .setColorValue(1)
    .setFont(createFont("AmericanTypewriter-CondensedLight", 6))
    ;
  cp5.setAutoDraw(false);
}

void draw(){
  textFont(font);
  frameRate(drawspeed);
  background((background / 10), background, background);
  colorMode(HSB, 100, 100, 100);
  
  Random rd = new Random();
  
  if (cp5.getWindow().isMouseOver()) {
    cam.setActive(false);
  } else {
    cam.setActive(true);
  }

  if (presentationMode) {
    rotateZ(frameCount*0.001);
    if (frameCount % 120 == 0) {
      if (rd.nextBoolean()) {
        Transcript = rd.nextBoolean();
        OCR = rd.nextBoolean();
        Keywords = rd.nextBoolean();
      }
      else if (rd.nextBoolean()) {
        Topics = rd.nextBoolean();
        Labels = rd.nextBoolean();
        namedLocations = rd.nextBoolean();
      }
      else if (rd.nextBoolean()) {
        namedPeople = rd.nextBoolean();
        audioEffects = rd.nextBoolean();
        Sentiments = rd.nextBoolean();
      }
      else {
        Emotions = rd.nextBoolean();
        Point = rd.nextBoolean();
        Line = rd.nextBoolean();
      }
    }
  }
  
  for (int i=0;i<insights.size();i++){
    insight theinsight = insights.get(i);
    float x, y, z;
    color c;
    
    x = theinsight.x * 10;
    y = theinsight.y * 10;
    z = theinsight.z * 10;

    if (Transcript==true & theinsight.insightType.equals("transcript")){
    }
    else if (OCR==true & theinsight.insightType.equals("ocr")){
    }
    else if (Keywords==true & theinsight.insightType.equals("keywords")){
    }
    else if (Topics==true & theinsight.insightType.equals("topics")){
    }
    else if (Labels==true & theinsight.insightType.equals("labels")){
    }
    else if (namedLocations==true & theinsight.insightType.equals("namedLocations")){
    }
    else if (namedPeople==true & theinsight.insightType.equals("namedPeople")){
    }
    else if (audioEffects==true & theinsight.insightType.equals("audioEffects")){
    }
    else if (Sentiments==true & theinsight.insightType.equals("sentiments")){
    }
    else if (Emotions==true & theinsight.insightType.equals("emotions")){
    }
    else{
      continue;
    }
    noFill();
    c = colorDetermine(theinsight.insightType);
    stroke(c);
    strokeWeight(.3);
    
    boolean first = true;
    
    boolean bool = true;
    
    boolean next = false;
    
    String previousinsightType = "";
    
    String currentinsightType = "";
    
    if (Line == true) {
      for (int j = 0; j < insights.size(); ++j) {
        if (theinsight.insight == insights.get(j).insight) {
          continue;
        }
        if (first) {
          previousinsightType = insights.get(j).insightType;
  
          first = false;
        } 
        currentinsightType = insights.get(j).insightType; 
          
        if (next) {
          if (currentinsightType != previousinsightType) {
            next = false;
          }
        } 
        else if (bool) {
          if ((theinsight.start < insights.get(j).end) & (insights.get(j).start < theinsight.end)) {

          line(x, y, z, insights.get(j).x * 10, insights.get(j).y * 10, insights.get(j).z * 10);
          }
        }
        else {
          bool = false;
          if ((theinsight.start < insights.get(j).end) & (insights.get(j).start < theinsight.end)) {

            line(x, y, z, insights.get(j).x * 10, insights.get(j).y * 10, insights.get(j).z * 10);
          }
          else {
            if (currentinsightType == previousinsightType) {
              next = true;
            }
            else {
              bool = true;
              
              next = false;
            }
          }
        }
        previousinsightType = currentinsightType;
      }
      /* curve( 
            x/3+x*2/3, y/3+y*2/3+20, z/3+z*2/3, 
            x, y, z, 
            x, y, z,
            x*2/3+x/3+20, y*2/3+y/3, z*2/3+z/3); */
    }
   
    if (Point==true){
      // insightwork
      //noStroke();
      //lights();
      pushMatrix();
      translate(x, y, z);
      c = colorDetermine(theinsight.insightType);
      stroke(c);
      //stroke(#000000);
      strokeWeight(7);
      point(0, 0, 0);
      //sphere(1);
      popMatrix();
      
      // insightist
      pushMatrix();
      // translate(x0, y0, z0);
      stroke(#000000);
      //stroke(#FF0000);
      strokeWeight(5);
      point(0, 0, 0);
      //sphere(1);
      popMatrix();
    }
    
    float p_x = screenX(x,y,z);
    float p_y = screenY(x,y,z);
    // calculate the distance between (p_x, p_y), (mouse_X, mouse_Y)
    float distanceObjectMouse = sq(p_x - mouseX) + sq(p_y - mouseY);
    if (distanceObjectMouse < 20){
      fill(#FFFFFF);
      stroke(#FFF0FB);
      strokeWeight(3);
      textSize(75);
      pushMatrix();
      scale(0.1);
      float[] rotations = cam.getRotations();
      rotateX(rotations[0]);
      rotateY(rotations[1]);
      rotateZ(rotations[2]);
      //text(theinsight.Title,0,0,0);
      text("InsightType: " + theinsight.insightType,x,y,z);
      text("Insight: " + theinsight.insight, x, y + 80, z);
      text("Confidence: " + theinsight.confidence, x, y + 160, z);
      text("Start: " + theinsight.stringstart, x, y + 240, z);
      text("End: " + theinsight.stringend, x, y + 320, z);
      textAlign(LEFT,CENTER);
      popMatrix();
    }
  }
  gui();
}


void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  textFont(font);
  textSize(15);
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}
